import java.security.*;
import javax.crypto.*;

public class fipstest {

    public static void main(String[] args) {
        if(isFIPSmode()) {
           throw new RuntimeException("FIPS mode enabled");
        }
        System.out.println("Regular mode");
    }

    // Test BCFIPS provider is active
    private static boolean isBCFIPSEnabled() {
        System.setProperty("org.bouncycastle.fips.approved_only", "true");
        if (org.bouncycastle.crypto.fips.FipsStatus.isReady()) {
            java.security.SecureRandom random = new java.security.SecureRandom();
            if (random.getProvider().getName().equals("BCFIPS")) {
               return true;
            } else {
               return false;
            }
        }
        return false;
    }

    // Test FIPS-140 mode enabled
    private static boolean isFIPSmode() {
        // the following ciphers are not allowed in the FIPS mode
        String[] FipsDisabledCiphers = new String[]{"RC2", "RC4", "DES", "Blowfish"};
        String[] AllowedCiphers = new String[]{"AES", "DESede"};
        boolean isFipsMode = true;

        for(String alg: FipsDisabledCiphers) {
            try {
                Cipher cipher = Cipher.getInstance(alg);
                // this cipher suite should be disabled in FIPS mode
                isFipsMode = false;
            } catch(NoSuchAlgorithmException | NoSuchPaddingException ex) {
                // expected exception in FIPS mode
            }
        }
        for(String alg: AllowedCiphers) {
            try {
                Cipher cipher = Cipher.getInstance(alg);
            } catch(NoSuchAlgorithmException | NoSuchPaddingException ex) {
                // this cipher suite should be enabled in FIPS mode
                isFipsMode = false;
            }
        }
        // 512-bit and 1024-bit key lengths are weak and not allowed in the FIPS mode
        int[] FipsDisabledKeyLen = new int[]{512, 1024};
        int[] AllowedKeyLen = new int[]{2048};

        for(int keyLen: FipsDisabledKeyLen) {
            try {
                KeyPairGenerator DSAkeyGen = KeyPairGenerator.getInstance("DSA");
                DSAkeyGen.initialize(keyLen);
                KeyPairGenerator RSAkeyGen = KeyPairGenerator.getInstance("RSA");
                RSAkeyGen.initialize(keyLen);
                // this cipher suite should be disabled in FIPS mode
                isFipsMode = false;
            } catch(NoSuchAlgorithmException | InvalidParameterException ex) {
                // expected exception in FIPS mode
            }
        }
        for(int keyLen: AllowedKeyLen) {
            try {
                KeyPairGenerator DSAkeyGen = KeyPairGenerator.getInstance("DSA");
                DSAkeyGen.initialize(keyLen);
                KeyPairGenerator RSAkeyGen = KeyPairGenerator.getInstance("RSA");
                RSAkeyGen.initialize(keyLen);
            } catch(NoSuchAlgorithmException | InvalidParameterException ex) {
                // this cipher suite should be enabled in FIPS mode
                isFipsMode = false;
            }
        }
        return isFipsMode;
    }
}